"""Module for working with colours."""

import inspect
import sys
from typing import Literal, Sequence, overload

import cmcrameri  # noqa
import matplotlib as mpl
import matplotlib.pyplot as plt
import numpy as np
import palettable  # noqa
import pywaffle


@overload
def create_colorlist(
    color_specifier: Sequence | str, n: int, map: Literal[False] = False
) -> list[str]: ...


@overload
def create_colorlist(
    color_specifier: Sequence | str, n: int, map: Literal[True]
) -> mpl.colors.Colormap: ...


def create_colorlist(
    color_specifier: Sequence | str, n: int, map: bool = False
) -> list[str] | mpl.colors.Colormap:
    """Create `n` colors from a color map.

    The `n` colours are drawn from one of the matplotlib colour maps, or a new colour
    map is created using the input colours. Specifying 'help' as the ``color_specifier``
    will print the list of available colour names in a nicer format.

    Parameters
    ----------
    color_specifier : Sequence | str
        The name of a matplotlib colour map or a list of two colors. To get the full
        list of available color maps, try with a nonsense color map, or see
        https://matplotlib.org/stable/tutorials/colors/colormaps.html#classes-of-colormaps.
        If two colors in a list is provided, any color that
        ``matplotlib.colors.LinearSegmentedColormap.from_list`` accepts is valid. You
        may also use the special name 'help' to get a list of available colour names.
    n : int
        The number of colors to be created.
    map : bool
        If True, return the colour map itself instead of the list. To draw colours from
        it, use ``color_map(range(n))``, or to place HEX values in a list::

            import matplotlib.colors as mpl.colors
            n = 5
            c_list = [mpl.colors.to_hex(c) for c in color_map(range(n))]

    Returns
    -------
    list[str] | mpl.colors.Colormap
        A list of `n` colors in HEX format.

    Raises
    ------
    AttributeError
        If the type of the ``color_specifier`` is not recognized.

    See Also
    --------
    `Palettable<https : //jiffyclub.github.io/palettable/#documentation>`_ is a collection
    of predefined colour maps that include the matplotlib colours, cmcrameri colours and
    many more (colour maps from the Wes Anderson movies being one of them!). Let us look
    at an example : :

        import matplotlib.pyplot as plt
        from palettable.wesanderson import Moonrise1_5 as wes_clr
        import plastik
        plastik.colors.make_color_swatch(
            plt.figure(figsize=(12, 3)).gca(),
            wes_clr.hex_colors,
        )

    cosmoplots.generate_hex_colors : Color generator based on standard colour maps.

    Note
    ----
    With `colour<https://github.com/vaab/colour>`_ we can do it like this::

        import colour
        c_grad = [
            c.get_hex()
            for c in list(colour.Color(start_c).range_to(colour.Color(main_c), n))
        ]

    But, the project is old (last commit on 19 Nov 2017), so let us do it with
    ``matplotlib.colors`` instead. The results are not identical, but who knows
    which is better.
    """
    if isinstance(color_specifier, list):
        return _create_colorlist_between(color_specifier, n, map=map)
    elif isinstance(color_specifier, str):
        return _create_colorlist_from(color_specifier, n, map=map)
    raise AttributeError("'color_specifier' must be either a list or a string.")


def _create_colorlist_from(cmap_name: str, n: int, map: bool = False) -> list[str]:
    """Create `n` colors that are drawn from one of the matplotlib color maps.

    Parameters
    ----------
    cmap_name : str
        The name of a matplotlib color map. To get the full list of available color
        maps, try with a nonsense color map, or see
        https://matplotlib.org/stable/tutorials/colors/colormaps.html#classes-of-colormaps.
    n : int
        The number of colors to be created.
    map : bool
        If True, return the color map itself instead of the list. To draw colors from
        it, use ``color_map(range(n))``, or to place HEX values in a list::

            import matplotlib.colors as mpl.colors
            n = 5
            c_list = [mpl.colors.to_hex(c) for c in color_map(range(n))]

    Returns
    -------
    list[str]
        A list of `n` colors in HEX format.
    """
    if cmap_name == "help":
        try:
            plt.get_cmap(cmap_name, 1)
        except Exception as e:
            s1, s2 = str(e).split(" supported values are ")
            print("Supported colour names are:")
            for word in s2.split(", "):
                word_ = word[1:-1]
                if word_.startswith("cmc"):
                    print("\t", word_, "\t(from cmcrameri)")
                else:
                    print("\t", word_)
        sys.exit()
    if map:
        return plt.get_cmap(cmap_name, n)
    return [mpl.colors.to_hex(c) for c in plt.get_cmap(cmap_name, n)(range(n))]


def _create_colorlist_between(colors: Sequence, n: int, map: bool = False) -> list[str]:
    """Create `n` colors between two colors (inclusive).

    Parameters
    ----------
    colors : Sequence
        The full sequence of colours we should create a colour list from. It should
        contain ``str`` objects, but we could have accepted more types here, since
        anything that ``matplotlib.colors.LinearSegmentedColormap.from_list`` accepts is
        valid.
    n : int
        The length of the list of colours from the colour map.
    map : bool
        If True, return the color map itself instead of the list. To draw colors from
        it, use ``color_map(range(n))``, or to place HEX values in a list::

            import matplotlib.colors as mpl.colors
            n = 5
            c_list = [mpl.colors.to_hex(c) for c in color_map(range(n))]

    Returns
    -------
    list[str]
        The hex values of all generated colors.
    """
    if map:
        return mpl.colors.LinearSegmentedColormap.from_list("Custom", colors, N=n)
    return [
        mpl.colors.to_hex(c)
        for c in mpl.colors.LinearSegmentedColormap.from_list("Custom", colors, N=n)(
            range(n)
        )
    ]


def palettable_help() -> None:
    """Print helt text about the `palettable` package."""
    print(
        "This package includes `palettable` as a dependency, but do not implement any"
        " of its colour maps. Rather, have a look at their documentation at"
        " https://jiffyclub.github.io/palettable/#documentation and use it as its own"
        " package. "
    )
    print(
        "Integrating it with `plastik` is simple, just load any palette, and for"
        " example call the `hex_colors` attribute they all have (again, see the"
        " `palettable` documentation). This is a list of HEX colours that can then be"
        " given to `make_color_swatch`."
    )
    print(
        "And just to tease you a bit more, here is a list of the available colour"
        " palette modules:"
    )
    for i in inspect.getmembers(palettable, inspect.ismodule):
        if i[0] not in ["utils", "palette"]:
            print("\t", i[0])


def make_color_swatch(  # noqa: PLR0913
    ax: plt.Axes,
    c_bar: mpl.colors.Colormap | list[str],
    vertical: bool = False,
    resolution: int = 90,
    ratio: int = 8,
    no_border: bool = False,
    no_ticks: bool = True,
) -> plt.Axes:
    """Create a color swatch on the given axis object from the given color map.

    Parameters
    ----------
    ax : plt.Axes
        The axis object to create the colour bar on.
    c_bar : mpl.colors.Colormap | list[str]
        A color map to draw colors from or a list of all colour that should be used.
    vertical : bool
        Whether the colours in the colour bar should be vertically or horizontally
        aligned. Default is False, a horizontal alignment. Only used when ``c_bar`` is a
        ``list``.
    resolution : int
        The horizontal resolution / density of the color gradient. Only used when
        ``cmap`` is a ``mpl.colors.Colormap``.
    ratio : int
        The vertical-to-horizontal ratio of one individual colour in the colour swatch.
        Only used when ``cmap`` is a ``mpl.colors.Colormap``. (The ratio can at the
        moment only be made 1:1, or slimmer). When ``cmap`` is a ``list``, the figure
        size or axis size defines the total width and height of the colour swatch.
    no_border : bool
        Remove all borders. Takes precedence over ``no_ticks``. Default is ``False``.
        See ``plt.Axes.set_axis_off``.
    no_ticks : bool
        Remove all tick marks, but keep the border. Default is ``True``. See
        ``plt.Axes.set_xticks`` and ``plt.Axes.set_yticks``.

    Returns
    -------
    plt.Axes
        Return the input axes object.

    Examples
    --------
    Let us create a figure that uses the function to create its colour bar. We here use
    a pre-defined colour palette for a section of the colour map, in addition to adding
    in some of our own colours.

    >>> viridis = plastik.colors.create_colorlist("viridis", 14)

    Now combine viridis with some random colours.

    >>> custom = viridis.copy()
    >>> for _ in range(3):
    ...     custom.append("#6543ff")
    >>> for _ in range(3):
    ...     custom.insert(0, "#2eff2e")

    Plot the ``color_map`` alone. The ``figsize`` parameter exactly defines the shape of
    the colour swatch.

    >>> plt.figure(figsize=(10, 1), layout="constrained")
    >>> plastik.colors.make_color_swatch(plt.gca(), custom, no_ticks=False)

    Then, as an inset with other data.

    >>> plt.figure()
    >>> ax = plt.gca()
    >>> x0, y0, width, height = 0.52, 0.8, 0.40, 0.15
    >>> ax.scatter(
    ...     np.arange(len(custom)),
    ...     np.ones(len(custom)),
    ...     c=custom,
    ... )
    >>> ax2 = ax.inset_axes([x0, y0, width, height])
    >>> ax2 = plastik.colors.make_color_swatch(
    ...     ax2, custom, ratio=2 * len(custom), no_border=True
    ... )

    You can make a custom label with the returned axis element's ``set_xticks`` (or
    ``set_yticks``), where ``length`` is the length of the given ``c_bar`` list or equal
    to the resolution.

    >>> length = len(custom)
    >>> ax2.set_xticks(list(range(length)), [f"No: {i}" for i in range(length)])
    >>> plt.show()
    """
    if isinstance(c_bar, list):
        row_col = "columns" if vertical else "rows"
        kwarg = {row_col: 1}
        pywaffle.Waffle.make_waffle(
            ax=ax,
            values=[1 for _ in range(len(c_bar))],
            colors=c_bar,
            interval_ratio_x=0,
            interval_ratio_y=0,
            starting_location="NW",
            vertical=True,
            block_arranging_style="snake",
            tight={"pad": 0},
            **kwarg,
        )
        ax.set_axis_on()
    else:
        gradient = np.linspace(0, 1, resolution)
        if resolution // ratio < 1:
            print(
                f"The resolution cannot be less than the ratio (={ratio}). Setting"
                f" {resolution = } -> {ratio}"
            )
            resolution = ratio
        gradient = np.vstack(tuple(gradient for _ in range(int(resolution // ratio))))
        ax.imshow(
            gradient,
            cmap=c_bar,
            interpolation="nearest",
            origin="lower",
        )
    if no_ticks:
        ax.set_xticks([])
        ax.set_yticks([])
    if no_border:
        ax.set_axis_off()
    return ax



import matplotlib.pyplot as plt
import numpy as np

from plastik import ridge


def r():
    # Data
    data = [
        (np.array([1, 2, 3]), np.array([6, 5, 7])),
        (np.array([2, 3, 4]), np.array([100, 1, -200])),
    ]
    lab = ["one", "two"]
    ridge.ridge_plot(data, "blank", "x_lim_S", "slalomaxis", "grid", "squeeze", labels=lab, lt="--", xlabel="XXX", ylabel="YYY")


if __name__ == "__main__":
    r()
    plt.show()



"""Manipulate the legend of a matplotlib figure."""

from typing import Any, Literal

import matplotlib as mpl
import numpy as np


def topside_legends(  # noqa: PLR0913
    ax: mpl.axes.Axes,
    *args: Any,
    c_max: int = 4,
    alpha: float = 0.8,
    side: Literal[
        "top",
        "bottom",
        "right",
        "left",
        "top right",
        "top left",
        "bottom right",
        "bottom left",
    ] = "top",
    edgecolor: str | tuple[float, float, float] = "",
    facecolor: str | tuple[float, float, float] = "",
    anchor_: tuple[float, float] | None = None,
    **kwargs: Any,
) -> mpl.axes.Axes:
    """Move the legend to the top of the plot.

    Parameters
    ----------
    ax : mpl.axes.Axes
        The axes object of the figure
    *args : Any
        Parameters given to ax.legend(), i.e. handles and labels. This is useful if you
        have many axes objects with one or more lines on them, but you want all lines in
        one legend at a given axes object.
    c_max : int
        Total number of columns allowed. If less than `c_max` labels are used, `c_max`
        is set to the number of labels. Defaults to 4.
    alpha : float
        Alpha value for the background of the legend. Defaults to 0.8.
    side : Literal['top', 'bottom', 'right', 'left', 'top right', 'top left', 'bottom right', 'bottom left']
        Places the legend at the given side. Valid sides are 'top', 'bottom', 'right',
        'left', 'top right', 'top left', 'bottom right' and 'bottom left'. Defaults to
        'top'.
    edgecolor : str | tuple[float, float, float]
        Set the colour of the legend edge. Can also be set with the alias 'ec'. Values
        can be either a string colour or a RGB tuple.
    facecolor : str | tuple[float, float, float]
        Set the colour of the legend face/background. Can also be set with the alias
        'fc'. Values can be either a string colour or a RGB tuple.
    anchor_ : tuple[float, float] | None, optional
        If you want to place the legend at a custom location, you can set the 'anchor'
        keyword parameter to a tuple. (0, 0) is the bottom left of the plot, (1, 1) is
        the top right. The anchor location defines where the 'side' parameter goes, so
        with 'anchor=(0.5, 0.5)' and 'side="top left"', top top left corner of the
        legend will be at the centre of the plot.
    **kwargs : Any
        All keyword arguments are sent to ax.legend().
        See https://matplotlib.org/stable/api/legend_api.html#matplotlib.legend.Legend
        for details.

    Returns
    -------
    mpl.axes.Axes
        Axes object with updated (topside) legend.

    Raises
    ------
    ValueError
        If the first parameter is a string type (should be an axis Artist).
    """
    _sides = {
        "top": "upper center",
        "bottom": "lower center",
        "right": "center right",
        "left": "center left",
        "top right": "upper right",
        "top left": "upper left",
        "bottom right": "lower right",
        "bottom left": "lower left",
    }
    _anchors = {
        "top": (0.5, 1.05),
        "bottom": (0.5, -0.05),
        "right": (1.04, 0.5),
        "left": (-0.04, 0.5),
        "top right": (1.04, 1.05),
        "top left": (-0.04, 1.05),
        "bottom right": (1.04, -0.05),
        "bottom left": (-0.04, -0.05),
    }
    loc = _sides[side]
    edgecolor = kwargs.pop("ec", edgecolor)
    facecolor = kwargs.pop("fc", facecolor)
    anchor = anchor_ or _anchors[side]
    if args and isinstance(args[0][0], str):
        raise ValueError(
            "The first args parameter must be a sequence of Artist, not str."
        )
    less_than_two = 2
    if len(args) < less_than_two:
        try:
            # If the labels are defined directly in the legend as a list, calling
            # ax.legend() will re-set it to an empty legend. Therefore, we grab the list
            # and re-set it when we update the legend object.
            legend1: mpl.legend.Legend = ax.get_legend()
            lst = [l_.get_text() for l_ in legend1.get_texts()]
            l_d = len(legend1.get_texts())
        except AttributeError:
            # If, however, the labels are set when creating the lines objects (e.g.
            # ax.plot(x, y, label="Label for (x, y) data")), we first make sure the
            # legend object is created by calling ax.legend(), then we check how many
            # labels exist in it. Calling ax.legend() will in this case preserve all
            # labels.
            ax.legend()
            legend2: mpl.legend.Legend = ax.get_legend()
            lst = []  # The empty list is falsy.
            l_d = len(legend2.get_texts())
    else:
        lst = list(args[1])
        l_d = len(args[1])
    n_row = int(np.ceil(l_d / c_max))
    n_col = 1
    while l_d > n_col * n_row:
        n_col += 1
    if args:
        leg = ax.legend(
            args[0],
            lst,
            loc=loc,
            bbox_to_anchor=anchor,
            bbox_transform=ax.transAxes,
            ncol=n_col,
            **kwargs,
        )
    elif lst:
        leg = ax.legend(
            lst,
            loc=loc,
            bbox_to_anchor=anchor,
            bbox_transform=ax.transAxes,
            ncol=n_col,
            **kwargs,
        )
    else:
        leg = ax.legend(
            loc=loc,
            bbox_to_anchor=anchor,
            bbox_transform=ax.transAxes,
            ncol=n_col,
            **kwargs,
        )
    if facecolor:
        leg.get_frame().set_facecolor(facecolor)
    if edgecolor:
        leg.get_frame().set_edgecolor(edgecolor)
    leg.get_frame().set_alpha(alpha)
    return ax



"""Manipulate the axis of matplotlib figures."""

import matplotlib as mpl
import matplotlib.pyplot as plt


def dark_theme(
    *ax: mpl.axes.Axes,
    fig: mpl.figure.Figure | None = None,
    keep_yaxis: bool = False,
) -> None:
    """Change plot style to fit a dark background.

    This is better in e.g. beamers with dark theme.

    Parameters
    ----------
    *ax : mpl.axes.Axes
        Send in any number of matplotlib axes and the changes will be applied to all
    fig : mpl.figure.Figure | None, optional
        The figure object that should be altered
    keep_yaxis : bool
        Keep the colour of the label along the vertical axis as is.
        Useful if a plot has y-labels that are coloured to match the plotted
        lines. Defaults to False.
    """
    for a in ax:
        if not keep_yaxis:
            a.tick_params(axis="both", colors="w")
            a.yaxis.label.set_color("w")
        else:
            a.tick_params(axis="x", colors="w")
        a.xaxis.label.set_color("w")
        plt.setp(a.spines.values(), color="gray")
    if fig is not None:
        fig.patch.set_alpha(0)



"""Library for creating beautiful and insightful visualizations."""

from importlib.metadata import version

from plastik import colors
from plastik.axes import *  # noqa:F401,F403
from plastik.legends import *  # noqa:F401,F403
from plastik.percentiles import percentiles
from plastik.ridge import *  # noqa:F401,F403

__version__ = version(__package__)
__all__ = ["colors", "percentiles"]



"""Creates a ridge plot figure."""

import itertools
from typing import Any, Dict, List, Optional, Tuple, Union

import attr
import matplotlib as mpl
import matplotlib.gridspec as grid_spec
import matplotlib.pyplot as plt
import numpy as np

import plastik


@attr.s(auto_attribs=True)
class Ridge:
    """Plot data in a ridge plot with fixed width and fixed height per ridge.

    Parameters
    ----------
    data : List
        A list of n 2-tuples with (x, y)-pairs; list of n np.ndarrays: (y)
    options : str
        String with characters that set different options. This include 'b' (blank), 'c'
        (crop x-axis), 'g' (grid), 's' (slalom axis) and 'z' (squeeze). Blank removes
        all axis lines and turns the grid off. Crop sets the x-axis to the smallest
        common x-limit in all data tuples. Grid turns on the grid. Slalom axis make the
        y-axis ticks alternate between the left and right side. Squeeze makes each axis
        plot overlap by 50% (this turns slalom axis on, and makes two RHS ticks
        contiguous). The options are combined in a single string in arbitrary order. Any
        other characters than 'bcgsz' can be included without having any effect, the
        class with just look for existence of any of 'bcgsz'.
    y_scale : float
        Scale of y-axis relative to the default which decides the total height of the
        figure. Defaults to 1.
    xlabel : str
        x-label placed at the bottom.
    ylabel : str
        y-label for all y-axis.
    ylim : List
        List containing the upper and lower y-axis limit in all ridges.
    pltype : str
        plt class (loglog, plot, semilogx etc.) Defaults to plot.
    kwargs : Dict
        Any keyword argument plt.plot accepts. (Need to be a dict, asterisk syntax not
        supported.)
    """

    data: List[Any] = attr.ib()

    @data.validator
    def _check_data_type(self, _, value):
        if not isinstance(value[0], tuple) and not isinstance(value[0], np.ndarray):
            raise TypeError(
                "data must be a list of tuples or numpy arrays, not list of"
                f" {type(self.data[0])}."
            )

    options: str = attr.ib(converter=str)
    y_scale: float = attr.ib(converter=float, default=1.0)
    xlabel: Optional[str] = attr.ib(converter=str, kw_only=True, default="")
    ylabel: Optional[str] = attr.ib(converter=str, kw_only=True, default="")
    xlim: List[float] = attr.Factory(list)
    ylim: List[float] = attr.Factory(list)
    pltype: str = attr.ib(converter=str, default="plot")
    kwargs: Dict[str, Any] = attr.Factory(dict)
    colors = itertools.cycle(plt.rcParams["axes.prop_cycle"].by_key()["color"])

    def set_grid(self) -> None:
        """Set the gridstructure of the figure."""
        fsize = (4, self.y_scale * len(self.data))
        self.gs = grid_spec.GridSpec(len(self.data), 1)
        self.__fig = plt.figure(figsize=fsize)
        # Set line type of horizontal grid lines
        self.gls = itertools.cycle(["-", "--"])
        self.ax_objs: List[plt.Axes] = []
        if "z" in self.options:
            self.gs.update(hspace=-0.5)
        else:
            self.gs.update(hspace=0.0)

    def set_xaxs(self) -> None:
        """Set the x-axis limits."""
        if self.xlim:
            x_min, x_max = self.xlim
        elif len(self.data[0]) != 2:  # noqa: PLR2004
            x_min, x_max = -0.5, len(self.data[0]) - 0.5
            x_min = 0.5 if self.pltype in ["loglog", "semilogx"] else x_min
        elif "c" in self.options:
            x_min, x_max = self.__x_limit(False)
        else:
            x_min, x_max = self.__x_limit()
        self.__xmin = x_min
        self.__xmax = x_max

    def set_ylabel(
        self, y_min: Optional[float] = None, y_max: Optional[float] = None
    ) -> None:
        """Set the y-axis label."""
        if y_min is None or y_max is None:
            self.ax = self.__fig.add_subplot(111, frame_on=False)
            self.ax.tick_params(
                labelcolor="w",
                axis="both",
                which="both",
                zorder=-1,  # labelleft=False,
                labelbottom=False,
                top=False,
                bottom=False,
                left=False,
                right=False,
            )
            plt.setp(self.ax.get_yticklabels(), alpha=0)
        else:
            self._set_ymin_ymax(y_min, y_max)

    def _set_ymin_ymax(self, y_min, y_max):
        self.ax.spines["top"].set_visible(False)
        self.ax.spines["bottom"].set_visible(False)
        self.ax.spines["left"].set_visible(False)
        self.ax.spines["right"].set_visible(False)
        if self.pltype != "plot":
            pltype = "log" if self.pltype in ["semilogy", "loglog"] else "linear"
            self.ax.set_yscale(pltype)
        self.ax.set_ylabel(self.ylabel)
        y_min = 1e-3 if self.pltype == "log" and y_min <= 0 else y_min
        ylim = self.ylim or [y_min, y_max]
        self.ax.set_ylim(ylim)

    def __blank(self) -> None:
        spine = ["top", "bottom", "left", "right"]
        for sp in spine:
            self.ax_objs[-1].spines[sp].set_visible(False)
        plt.tick_params(
            axis="both",
            which="both",
            bottom=False,
            left=False,
            top=False,
            right=False,
            labelbottom=False,
            labelleft=False,
        )

    def __z_option(self, i) -> None:
        if i % 2:
            self.ax_objs[-1].tick_params(
                axis="y",
                which="both",
                left=False,
                labelleft=False,
                labelright=True,
            )
            self.ax_objs[-1].spines["left"].set_color("k")
        else:
            self.ax_objs[-1].tick_params(
                axis="y",
                which="both",
                right=False,
                labelleft=True,
                labelright=False,
            )
            self.ax_objs[-1].spines["right"].set_color("k")

    def __s_option(self, i) -> None:
        if i % 2:
            self.ax_objs[-1].tick_params(
                axis="y", which="both", labelleft=False, labelright=True
            )

    def __g_option(self, i) -> None:
        if ("g" in self.options and "z" not in self.options) or (
            "g" in self.options and len(self.data) == 1
        ):
            plt.grid(True, which="major", ls="-", alpha=0.2)
        elif "g" in self.options:
            plt.minorticks_off()
            alpha = 0.2 if i in (0, len(self.data) - 1) else 0.1
            plt.grid(True, axis="y", which="major", ls=next(self.gls), alpha=0.2)
            plt.grid(True, axis="x", which="major", ls="-", alpha=alpha)

    def __resolve_first_last_axis(self, i) -> None:
        if i == len(self.data) - 1:
            if self.xlabel:
                plt.xlabel(self.xlabel)
            if len(self.data) != 1:
                plt.tick_params(axis="x", which="both", top=False)
        elif i == 0:
            plt.tick_params(
                axis="x", which="both", bottom=False, labelbottom=False
            )  # , labeltop=True
        else:
            plt.tick_params(
                axis="x", which="both", bottom=False, top=False, labelbottom=False
            )

    def __resolve_options(self, i, spines, col) -> None:
        if len(self.data) != 1:
            if "z" in self.options:  # Squeeze
                self.__z_option(i)
            elif "s" in self.options:  # Slalom axis
                self.__s_option(i)
            for sp in spines:
                self.ax_objs[-1].spines[sp].set_visible(False)
            if "z" not in self.options:  # Squeeze
                self.ax_objs[-1].spines["left"].set_color(col)
                self.ax_objs[-1].spines["right"].set_color(col)
            self.ax_objs[-1].tick_params(axis="y", which="both", colors=col)
            self.ax_objs[-1].yaxis.label.set_color(col)
        self.__g_option(i)
        self.__resolve_first_last_axis(i)

    def __setup_axis(
        self,
        y_min: float,
        y_max: float,
        i: int,
        s: Union[Tuple[np.ndarray, np.ndarray], np.ndarray],
    ) -> Tuple[
        float, float, Union[Tuple[np.ndarray, np.ndarray], np.ndarray], List[str]
    ]:
        self.ax_objs.append(self.__fig.add_subplot(self.gs[i : i + 1, 0:]))
        if i == 0:
            spines = ["bottom"]
        elif i == len(self.data) - 1:
            spines = ["top"]
        else:
            spines = ["top", "bottom"]
        s_ = s if isinstance(s, np.ndarray) else s[1]
        y_min = min(s_.min(), y_min)
        y_max = max(s_.max(), y_max)
        return y_min, y_max, s, spines

    def __draw_lines(self, s, col) -> None:
        # Plot data
        p_func = getattr(self.ax_objs[-1], self.pltype)
        if len(s) == 2:  # noqa: PLR2004
            ell = p_func(s[0], s[1], color=col, markersize=2.5, **self.kwargs)[0]
        else:
            ell = p_func(s, color=col, markersize=2.5, **self.kwargs)[0]

        # Append in line-list to create legend
        self.__lines.append(ell)

    def data_loop(self) -> Tuple[float, float]:
        """Run the data loop."""
        # Loop through data
        self.__lines: List[plt.Line2D] = []
        y_min = np.inf
        y_max = -np.inf
        for i, s in enumerate(self.data):
            col = next(self.colors)
            y_min, y_max, s_, spines = self.__setup_axis(y_min, y_max, i, s)
            self.__draw_lines(s_, col)
            self.ax_objs[-1].patch.set_alpha(0)
            # Scale all subplots to the same x axis
            plt.xlim([self.__xmin, self.__xmax])
            if self.ylim:
                plt.ylim(self.ylim)

            # The length of data is greater than one, fix the plot according to the
            # input args and kwargs.
            if "b" in self.options:
                self.__blank()
            else:
                self.__resolve_options(i, spines, col)
        return y_min, y_max

    def __x_limit(self, maxx=True) -> Tuple[float, float]:
        if isinstance(self.data[0], tuple):
            data: List[np.ndarray] = [d[0] for d in self.data]
        else:
            raise ValueError("'data' must have x-values.")
        t_min = data[0]
        x_max = data[0][-1]
        for t in data[1:]:
            t_0, t_max = np.min(t), np.max(t)
            if maxx:
                t_min = t if t_0 < t_min[0] else t_min
                # t_max = t if t_1 > t_max[-1] else t_max
                x_max = max(t_max, x_max)
            else:
                t_min = t if t[0] > t_min[0] else t_min
                # t_max = t if t[-1] < t_max[-1] else t_max
                x_max = min(t_max, x_max)
        diff = 0.05 * (x_max - t_min[0])
        # x_max = t_max[-1] + diff
        x_max += diff
        if self.pltype in ["loglog", "semilogx"]:
            x_min = 0.8 * t_min[t_min > 0][0] if t_min[0] < diff else t_min[0] - diff
            # if x_min < 0:
            #     x_min = 1e-10
        else:
            x_min = t_min[0] - diff
        return x_min, x_max

    @property
    def lines(self) -> List:
        """Return the plotted lines."""
        return self.__lines

    @property
    def figure(self) -> mpl.figure.Figure:
        """Return the figure."""
        return self.__fig

    @property
    def top_axes(self) -> mpl.axes.Axes:
        """Return the top axes."""
        return self.ax_objs[0]

    @property
    def bottom_axes(self) -> mpl.axes.Axes:
        """Return the bottom axes."""
        return self.ax_objs[-1]

    @property
    def ylabel_axis(self) -> mpl.axes.Axes:
        """Return axis with y-label."""
        return self.ax

    @property
    def all_axes(self) -> List[mpl.axes.Axes]:
        """Return all the axes."""
        return self.ax_objs

    def main(self) -> None:
        """Run the main function."""
        self.set_grid()
        self.set_xaxs()
        if self.ylabel:
            self.set_ylabel()
        y1, y2 = self.data_loop()
        if self.ylabel:
            self.set_ylabel(y1, y2)


if __name__ == "__main__":
    x = np.linspace(1e-1, 3e1, 1000) ** 2

    def _func(x, s):
        return 10 / ((x - s) ** 2 + 1)

    dt = [
        _func(x, 3),
        _func(x, 1),
        _func(x, 0),
        _func(x, 100),
        _func(x, 300),
        _func(x, 500),
    ]
    dta = [(x, a) for a in dt]
    lab = [f"{i}" for i in range(6)]
    kws = {"ls": "-."}
    r = Ridge(dt, "z", ylabel="xlabel", pltype="semilogx", y_scale=0.5)
    r.main()
    f = r.figure
    ell = r.lines
    at = r.top_axes
    ab = r.bottom_axes
    axs = r.all_axes
    at.legend(ell, lab)
    plastik.topside_legends(ab, ell, lab, c_max=2, side="bottom left")
    # for ax in axs:
    #     plastik.log_tick_format(ax, which="x")
    plt.show()



# plastik

# General imports
import matplotlib as mpl
import matplotlib.pyplot as plt
import numpy as np
from palettable.wesanderson import FantasticFox2_5

import plastik

SAVEDIR = "examples/figures"

# Set the plotting style using the `default.mplstyle` file.
mpl.style.use("plastik.default")
# Log tick format -------------------------------------------------------------------- #


def log_tick_format():
    # 1
    y = np.exp(np.linspace(-3, 5, 100))
    base = 2  # Default is 10, but 2 works equally well
    plt.figure()
    plt.semilogx(base=base)
    plt.plot(y)
    plt.savefig(f"{SAVEDIR}/log_tick_format1.png")
    plastik.dark_theme(plt.gca(), fig=plt.gcf())
    plt.savefig(f"{SAVEDIR}/log_tick_format1_dark.png")
    plt.show()
    plt.close("all")

    # 2
    y = np.exp(np.linspace(-3, 5, 100))
    base = 2  # Default is 10, but 2 works equally well
    fig = plt.figure()
    ax = fig.add_subplot(111)
    ax.loglog(base=base)
    ax.plot(y)
    plt.savefig(f"{SAVEDIR}/log_tick_format2.png")
    plastik.dark_theme(ax, fig=fig)
    plt.savefig(f"{SAVEDIR}/log_tick_format2_dark.png")
    plt.show()
    plt.close("all")

    # 3
    y = np.exp(np.linspace(-3, 5, 100))
    fig = plt.figure()
    ax = fig.add_subplot(111)
    ax.semilogy(y)
    plt.savefig(f"{SAVEDIR}/log_tick_format3.png")
    plastik.dark_theme(ax, fig=fig)
    plt.savefig(f"{SAVEDIR}/log_tick_format3_dark.png")
    plt.show()
    plt.close("all")


# Topside legends -------------------------------------------------------------------- #


def topside_legends():
    # 1
    y = np.linspace(-3, 5, 100)
    fig = plt.figure()
    ax = fig.add_subplot(111)
    ax.plot(y, label="Topside legend 1")
    ax.plot(y + 1, label="Topside legend 2")
    ax.plot(y + 2, label="Topside legend 3")
    ax.plot(y + 3, label="Topside legend 4")
    plastik.topside_legends(ax, c_max=2, side="bottom", alpha=0.2)
    plt.savefig(f"{SAVEDIR}/topside_legends1.png")
    plastik.dark_theme(ax, fig=fig)
    plt.savefig(f"{SAVEDIR}/topside_legends1_dark.png")
    plt.show()
    plt.close("all")

    # 2
    y = np.linspace(-3, 5, 100)
    fig = plt.figure()
    ax = fig.add_subplot(111)
    ax.plot(y, label="Topside legend 1")
    ax.plot(y + 1, label="Topside legend 2")
    ax.plot(y + 2, label="Topside legend 3")
    ax.plot(y + 3, label="Topside legend 4")
    plastik.topside_legends(ax, c_max=3, side="top right", alpha=1)
    plt.savefig(f"{SAVEDIR}/topside_legends2.png")
    plastik.dark_theme(ax, fig=fig)
    plt.savefig(f"{SAVEDIR}/topside_legends2_dark.png")
    plt.show()
    plt.close("all")


# Ridge ------------------------------------------------------------------------------ #


def ridge():
    # Set up
    x = np.linspace(1e-1, 3e1, 1000) ** 2

    def func(x, s):
        return 10 / ((x - s) ** 2 + 1)

    dt = [func(x, 3), func(x, 1), func(x, 0), func(x, 100), func(x, 300), func(x, 500)]
    dta = [(x, y) for y in dt]

    lab = [f"{i}" for i in range(6)]

    # 1
    r = plastik.Ridge(dta, "gsz", xlabel="bottom", ylabel="xlabel")
    r.main()
    f = r.figure
    ell = r.lines
    a = r.top_axes
    axs = r.all_axes
    a.legend(ell, lab)
    plt.savefig(f"{SAVEDIR}/ridge1.png")
    plastik.dark_theme(r.bottom_axes, keep_yaxis=True, fig=f)
    plastik.dark_theme(r.ax)
    plt.savefig(f"{SAVEDIR}/ridge1_dark.png")
    plt.show()
    plt.close("all")

    # 2
    r = plastik.Ridge(dta, "gs", ylabel="xlabel")
    r.main()
    f = r.figure
    ell = r.lines
    a = r.top_axes
    axs = r.all_axes
    a.legend(ell, lab)
    plastik.topside_legends(a, ell, c_max=6, side="right")
    for ax in axs:
        ax.semilogy()
    plt.savefig(f"{SAVEDIR}/ridge2.png")
    plastik.dark_theme(r.bottom_axes, keep_yaxis=True, fig=f)
    plastik.dark_theme(r.ax)
    plt.savefig(f"{SAVEDIR}/ridge2_dark.png")
    plt.show()
    plt.close("all")

    # 3
    r = plastik.Ridge(
        dta, "s", xlabel="bottom axis label", ylabel="xlabel", pltype="semilogx"
    )
    r.main()
    f = r.figure
    ell = r.lines
    a = r.top_axes
    axs = r.all_axes
    a.legend(ell, lab)
    plastik.topside_legends(a, ell, c_max=5, side="right")
    for ax in axs:
        ax.semilogx()
    plt.savefig(f"{SAVEDIR}/ridge3.png")
    plastik.dark_theme(r.bottom_axes, keep_yaxis=True, fig=f)
    plastik.dark_theme(r.ax)
    plt.savefig(f"{SAVEDIR}/ridge3_dark.png")
    plt.show()
    plt.close("all")

    # 4
    r = plastik.Ridge(dta, "bz", ylabel="xlabel", pltype="loglog")
    r.main()
    f = r.figure
    a = r.bottom_axes
    axs = r.all_axes
    for ax in axs:
        ax.loglog()
    plt.savefig(f"{SAVEDIR}/ridge4.png")
    plastik.dark_theme(r.bottom_axes, keep_yaxis=True, fig=f)
    plastik.dark_theme(r.ax)
    plt.savefig(f"{SAVEDIR}/ridge4_dark.png")
    plt.show()
    plt.close("all")


# Dark theme ------------------------------------------------------------------------- #


def dark_theme():
    # 1
    y = np.exp(np.linspace(-3, 5, 100))
    plt.figure()
    # Sets axes and labels of given axis to white
    plastik.dark_theme(plt.gca())
    plt.loglog(base=2)
    plt.plot(y)
    plt.xlabel("white label")
    plt.ylabel("ylabel")
    plt.savefig(f"{SAVEDIR}/dark_theme.png")
    plt.show()
    plt.close("all")


# Color map -------------------------------------------------------------------------- #
def color_map():
    plastik.colors.palettable_help()
    plastik.colors.make_color_swatch(
        plt.figure(figsize=(12, 3)).gca(),
        FantasticFox2_5.hex_colors,
    )
    prints = plastik.colors.create_colorlist("gist_rainbow", 20)
    print(prints)
    color_map = plastik.colors.create_colorlist("gist_rainbow", 20, map=True)
    # First 5 colors in the map
    first_five = [mpl.colors.to_hex(c) for c in color_map(range(5))]
    print(first_five)
    # All 20 colors in the map, with the last one repeated 30 times
    print([mpl.colors.to_hex(c) for c in color_map(range(50))])
    # All colors in the map
    color_list = [mpl.colors.to_hex(c) for c in color_map(range(20))]
    print(all(j == k for j, k in zip(prints, color_list)))  # True
    # A new map based on raw input (first five from gist_rainbow, n=20) is created and
    # samples are drawn from it.
    print(first_five)
    print(plastik.colors.create_colorlist(first_five, 20))

    custom_colors = ["#123456", "#654321", "#fedcba", "#abcdef", "#aa11aa", "#11aa11"]
    custom_colors_map = plastik.colors.create_colorlist(custom_colors, 20, map=True)
    print([mpl.colors.to_hex(c) for c in custom_colors_map(range(20))])

    # Plot the colors
    # As a color swatch 1
    plt.figure()
    plastik.colors.make_color_swatch(plt.gca(), color_map)
    # As a color swatch 2
    plt.figure()
    plastik.colors.make_color_swatch(plt.gca(), color_map, no_ticks=True, resolution=7)
    # As a color swatch 3
    plt.figure()
    plastik.colors.make_color_swatch(plt.gca(), color_map, no_border=True, ratio=50)
    # As an inset with other data
    plt.figure()
    ax = plt.gca()
    x0, y0, width, height = 0.52, 0.8, 0.40, 0.25
    ax.scatter(
        np.arange(20),
        np.ones(20),
        c=custom_colors_map(range(20)),
    )
    ax2 = ax.inset_axes([x0, y0, width, height])
    ax2 = plastik.colors.make_color_swatch(
        ax2, custom_colors_map, no_ticks=False, ratio=5
    )
    ax2.set_xticks([0, 30, 60, 90], [0, "hey", True, 2], size=6)
    ax2.set_yticks([])
    plt.show()

    # And as our final trick, let us use a pre-defined colour palette for a section of
    # the colour map, in addition to adding in some of our own.
    viridis = plastik.colors.create_colorlist("cmc.batlow", 14)
    v_check = plastik.colors.create_colorlist(viridis, 14)
    v_check2 = plastik.colors.create_colorlist([viridis[0], viridis[-1]], 14)
    print(viridis)  # True viridis (by definition)
    print(v_check)  # True viridis (equivalent to above)
    print(v_check2)  # Incorrect (fills in between according to a different method)
    # Now combine viridis with some random colours.
    custom = viridis.copy()
    for _ in range(3):
        custom.append("#6543ff")
    for _ in range(3):
        custom.insert(0, "#2eff2e")
    # Plot the color_map alone.
    plt.figure(figsize=(10, 1), layout="constrained")
    plastik.colors.make_color_swatch(plt.gca(), custom, no_ticks=False)
    # As an inset with other data
    plt.figure()
    ax = plt.gca()
    x0, y0, width, height = 0.57, 0.87, 0.40, 0.1
    ax.scatter(
        np.arange(len(custom)),
        np.ones(len(custom)),
        c=custom,
    )
    ax2 = ax.inset_axes([x0, y0, width, height])
    plastik.colors.make_color_swatch(ax2, custom, ratio=2 * len(custom))
    ax2.set_xticks([0, 3, 6, 17, 20], [0, "Zzz", "See", "Stop", 2], rotation=20, size=6)
    plt.show()


if __name__ == "__main__":
    log_tick_format()
    topside_legends()
    ridge()
    dark_theme()
    color_map()



"""Module for plotting percentiles from an ensemble of arrays."""

from typing import Any

import matplotlib as mpl
import matplotlib.pyplot as plt
import numpy as np


def percentiles(  # noqa: PLR0913
    x: np.ndarray,
    y: np.ndarray,
    n: int = 20,
    ax: mpl.axes.Axes | None = None,
    plot_mean: bool = False,
    plot_median: bool = True,
    **kwargs: Any,
) -> mpl.axes.Axes:
    """Calculate percentiles from ensemble 'y' along 'x'.

    Parameters
    ----------
    x : np.ndarray
        One dimensional array, x-axis.
    y : np.ndarray
        Values along y-axis. Need shape (N, len(x)).
    n : int
        The number of percentiles, linearly spaced from 50 to 'percentile_m{in,ax}'.
        Defaults to 20.
    ax : mpl.axes.Axes | None, optional
        The axes object to plot on. If not given, the current axes will be used.
    plot_mean : bool
        Plot mean of 'y'
    plot_median : bool
        Plot median of 'y'
    **kwargs : Any
        alpha : str
            Alpha value of each layer of the percentiles
        color : str
            Colour of the percentile shading. Can be any colour that can be parsed by
            matplotlib's plotting function.
        line_color : str
            Colour of the mean/median plot. Can be any colour that can be parsed by
            matplotlib's plotting function.
        percentile_min : float
            Lower percentile limit.
        percentile_max : float
            Upper percentile limit.

    Returns
    -------
    mpl.axes.Axes
        The axes object of the figure.
    """
    ax = ax or plt.gca()
    # calculate the lower and upper percentile groups, skipping 50 percentile
    percentile_min = kwargs.pop("percentile_min") if "percentile_min" in kwargs else 1
    percentile_max = kwargs.pop("percentile_max") if "percentile_max" in kwargs else 99
    perc1 = np.percentile(
        y, np.linspace(percentile_min, 50, num=n, endpoint=False), axis=0
    )
    perc2 = np.percentile(y, np.linspace(50, percentile_max, num=n + 1)[1:], axis=0)

    color = kwargs.pop("color") if "color" in kwargs else "r"
    line_color = kwargs.pop("line_color") if "line_color" in kwargs else "k"
    if plot_mean:
        plt.plot(x, np.mean(y, axis=0), color=line_color)

    if plot_median:
        plt.plot(x, np.median(y, axis=0), "-d", color=line_color)

    alpha = kwargs.pop("alpha") if "alpha" in kwargs else 1 / n
    # fill lower and upper percentile groups
    for p1, p2 in zip(perc1, perc2, strict=True):
        plt.fill_between(x, p1, p2, alpha=alpha, color=color, edgecolor=None)

    return ax
